<?php

namespace Pinpoint\Feefo\Block\Services;

use Magento\Framework\View\Element\Template;

/**
 * Class Reviews
 * @package Pinpoint\Feefo\Block\Services
 */
class Reviews extends Template
{
    /**
     * @var \Magento\Framework\Registry
     */
    protected $registry;

    /**
     * @var \Pinpoint\Feefo\Model\FeefoReview|\Pinpoint\Feefo\Model\FeefoServiceReview
     */
    protected $feefoReviews;

    /**
     * @var \Pinpoint\Feefo\Model\ResourceModel\FeefoServiceReview\Collection
     */
    protected $reviewCollection;

    /**
     * @var \Pinpoint\Feefo\Model\ResourceModel\FeefoServiceReview\Collection
     */
    protected $reviews;

    /**
     * @var \Pinpoint\Feefo\Helper\Data
     */
    protected $feefoHelper;

    /**
     * @var string
     */
    const SLIDER_ENABLED_CONFIG_PATH = 'pinpoint_feefo/service_widget/enabled';

    /**
     * Reviews constructor.
     * @param \Pinpoint\Feefo\Model\FeefoServiceReview $feefoReviews
     * @param \Magento\Framework\Registry $registry
     * @param Template\Context $context
     * @param array $data
     */
    public function __construct(
        \Pinpoint\Feefo\Model\FeefoServiceReview $feefoReviews,
        \Pinpoint\Feefo\Helper\Data $feefoHelper,
        \Magento\Framework\Registry $registry,
        Template\Context $context,
        array $data = []
    ) {
        $this->feefoReviews = $feefoReviews;
        $this->registry = $registry;
        $this->reviews = null;
        $this->reviewCollection = null;
        $this->feefoHelper = $feefoHelper;

        parent::__construct($context, $data);
    }

    /**
     * @param bool $forceUpdate
     * @return mixed|null|\Pinpoint\Feefo\Model\ResourceModel\FeefoServiceReview\Collection
     */
    public function getServiceReviews($forceUpdate = false)
    {
        if ($this->reviews && !$forceUpdate) {
            return $this->reviews;
        }

        $this->reviewCollection = $reviews = $this->feefoReviews->getCollection();
        if ($reviews->count()) {
            $this->reviews = $reviews;
            return $reviews;
        }

        return null;
    }

    /**
     * @return mixed|null|\Pinpoint\Feefo\Model\ResourceModel\FeefoServiceReview\Collection
     */
    public function getSliderReviews()
    {
        return $this->feefoReviews
            ->getCollection()
            ->setPageSize(5)
            ->setCurPage(1)
            ->setOrder('created_at', 'DESC');
    }

    /**
     * @return int
     */
    public function getReviewCount()
    {
        if (!$this->reviewCollection) {
            $this->getServiceReviews();
        }

        return $this->reviewCollection->count();
    }

    /**
     * @return float|int
     */
    public function getAverageRating()
    {
        if (!$this->reviewCollection) {
            $this->getServiceReviews();
        }

        $ratings = [
            '0' => 0,
            '1' => 0,
            '2' => 0,
            '3' => 0,
            '4' => 0,
            '5' => 0
        ];

        foreach ($this->reviewCollection as $review) {
            $ratings[$review->getData('rating')] += 1;
        }

        if (!array_sum(array_values($ratings))) {
            return 0;
        }

        $rating = (5*$ratings['5'] + 4*$ratings['4'] + 3*$ratings['3'] + 2*$ratings['2'] + 1*$ratings['1']) / array_sum(array_values($ratings));
        return floor($rating * 2) / 2;
    }

    /**
     * @return bool
     */
    public function isEnabled()
    {
        return (bool)$this->feefoHelper->getConfig(self::SLIDER_ENABLED_CONFIG_PATH);
    }

}